/**
  ******************************************************************************
  * @file    main.c
  * @author  MCD Application Team
  * @version V4.0.0
  * @date    21-January-2013
  * @brief   Virtual Com Port Demo main file
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2013 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  ******************************************************************************
  */

/*
 *   CDC Example ported to Coocox IDE 1.7.5 by SOC Robotics, Inc.  Nov 2013
 */
/* Includes ------------------------------------------------------------------*/
#include "hw_config.h"
//#include "usbd_cdc_core.h"
//#include "usbd_usr.h"
#include "usb_lib.h"
#include "usb_desc.h"
#include "usb_pwr.h"
#include "usbd_cdc_vcp.h"

#include "string.h"
#include "math.h"
#include "stdarg.h"
#include "stdio.h"
#include "stdlib.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Extern variables ----------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
#define RXBUFSIZE 100
#define 	XON   		  0x11
#define 	XOFF  		  0x13
#define 	LF	  		  0x0A
#define 	CR	  		  0x0D
#define  	SPACE 	   	  0x20

char rxbuf[RXBUFSIZE];
int rxheadptr,rxtailptr;

extern int printfu(const char *fmt, ...);

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/

/*******************************************************************************
* Function Name  : main.
* Description    : Main routine.
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
extern void PutChar(char inchar);

/* Private variables ---------------------------------------------------------*/
  RCC_ClocksTypeDef RCC_Clocks;
  uint32_t TimingDelay1 = 0,TimingDelay2=0,TimingDelay3=0,TimingDelay4=0;

/* Private function prototypes -----------------------------------------------*/
void TimingDelay_Decrement(void);
void Delay(uint32_t nTime);

/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Decrements the TimingDelay variable.
  * @param  None
  * @retval None
  */
void TimingDelay_Decrement(void)
{
  if (TimingDelay1 != 0x00) TimingDelay1--;
  if (TimingDelay2 != 0x00) TimingDelay2--;
  if (TimingDelay3 != 0x00) TimingDelay3--;
  if (TimingDelay4 != 0x00) TimingDelay4--;
}

/**
  * @brief  Inserts a delay time.
  * @param  nTime: specifies the delay time length, in 10 ms.
  * @retval None
  */
void Delay(uint32_t nTime)
{
  TimingDelay1 = nTime;
  printf("\n\r");
  while(TimingDelay1 != 0)  printf("%d   \r",TimingDelay1);;
}

int __errno(void)
 {
	 int i;

	 i += 1;
 }

char GetCharnw(void)
{
	char inchar;
	int tempptr;

	if(rxheadptr==rxtailptr)  return 0;
	inchar = rxbuf[rxtailptr];
	rxtailptr++;
	if(rxtailptr==RXBUFSIZE) rxtailptr = 0;
	return inchar;
}

char GetChar(void)
{
	char inchar;

	while(!(inchar=GetCharnw())) ;
	return inchar;
}

void Puts(char *inchar)
{
	int i;

	for(i=0; i<strlen(inchar); i++) PutChar(inchar[i]);
}


int printf (const char *fmt, ...)
{
	va_list args;
	int i,j;
	char printbuffer[250];

	va_start (args, fmt);
	i = vsnprintf (printbuffer, 250, fmt, args);
	va_end (args);

	// Print the string
	j = 0;
	while(printbuffer[j]!=0) PutChar(printbuffer[j++]);

	return i;
}

int scanf(const char *fmt, ...)
{
	char inchar;
	va_list args;
	int i,ptr;
	char scanbuffer[120];

	//printf("\n\rFormat: %s  args=%d",fmt,args);

	ptr = 0;
	while((inchar=GetChar())!=CR) {
        scanbuffer[ptr++] = inchar;
		//putchar(inchar);
		}
	scanbuffer[ptr] = 0;

	va_start (args, fmt);
	i = vsscanf(scanbuffer, fmt, args);
	va_end (args);

	return i;
}

int main(void)
{
	char inchar,outstrg[100];
	int i;
	long j;
	volatile float a,b,c,elapsedtime,angle,angleinc,radius;

	Set_System();
	Set_USBClock();
	USB_Interrupts_Config();
	// USB Device Initialize
	//USBD_Init(&USB_OTG_dev,
	//        USB_OTG_FS_CORE_ID,
	//        &USR_desc,
	//        &USBD_CDC_cb,
	//        &USR_cb);
	USB_Init();

	/* SysTick end of count event each 10ms */
	RCC_GetClocksFreq(&RCC_Clocks);
	SysTick_Config(RCC_Clocks.HCLK_Frequency / 100);

	rxtailptr = rxheadptr = 0;
	a = 0.125;

    while (1)  {
	  inchar = GetCharnw();
	  switch (inchar) {
	    case 's':
		  printf("\n\rF3 CDC Demo Example V1.00\n\r");
		  printf("Ported to CooCox CoIDE 1.7.5 by SOC Robotics, Inc.\n\r>");
		  //strcpy(outstrg,dtoa(s,a));
	  	  break;
	    case CR:
		  printf("\n\r>");
		  break;
	    case 'd':
		  printf("\n\rStart 2 second delay");
		  Delay(200);
		  printf(" - end\n\r>");
		  break;
	    case 'm':
		  printf("\n\rDo 10000 circular interpolation calculations\n\r");
		  TimingDelay4 = 10000;
		  angle = 0.125;
		  radius = 2.56;
		  angleinc = 0.002;
		  for(i=0; i<10000; i++) {
			  a = radius * sinf(angle);
			  b = radius * cosf(angle);
			  angle += angleinc;
			  }
		  elapsedtime = ((float)(10000 - TimingDelay4))/100.0;
		  printf("Single precision finished in %f seconds or %f usec/loop",elapsedtime,(elapsedtime*1000000.0)/10000.0);
		  TimingDelay4 = 10000;
		  angle = 0.125;
		  for(i=0; i<10000; i++) {
			  a = radius * sin(angle);
			  b = radius * cos(angle);
			  angle += angleinc;
			  }
		  elapsedtime = ((float)(10000 - TimingDelay4))/100.0;
		  printf("\n\rDouble precision finished in %f seconds or %f usec/loop\n\r>",elapsedtime,(elapsedtime*1000000.0)/10000.0);
		  break;
	    case 't':
		  printf("\n\rDo 10000 sin functions\n\r");
		  TimingDelay4 = 20000;
		  for(i=0; i<10000; i++) {
			  a += (0.005 + ((float)i)/10000000.0);
			  b = sinf(a);
			  }
		  elapsedtime = ((float)(20000 - TimingDelay4))/100.0;
		  //printf(" timingdelay4=%d\n\r",TimingDelay4);
		  printf(" %f %f finished in %f seconds or %f usec/loop\n\r>",a,b,elapsedtime,(elapsedtime*1000000.0)/10000.0);
		  break;
		}
  	  }
}
#ifdef USE_FULL_ASSERT
/*******************************************************************************
* Function Name  : assert_failed
* Description    : Reports the name of the source file and the source line number
*                  where the assert_param error has occurred.
* Input          : - file: pointer to the source file name
*                  - line: assert_param error line source number
* Output         : None
* Return         : None
*******************************************************************************/
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {}
}
#endif

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
